@@ -1,7 +1,7 @@
 #
 #    util.S    miscellaneous utilities
 #
-#    $Revision: 1.125 $    $Date: 2013/11/29 01:50:26 $
+#    $Revision: 1.148 $    $Date: 2014/05/06 15:32:53 $
 #
 #
 matrowsum <- function(x) {
@@ -151,14 +151,15 @@ pointgrid <- function(W, ngrid) {
 
 # text magic
 
-commasep <- function(x, join="and") {
+commasep <- function(x, join=" and ", flatten=TRUE) {
   px <- paste(x)
   nx <- length(px)
   if(nx <= 1) return(px)
   commas <- c(rep(", ", length(px)-2),
-              paste("", join, ""),
+              join,
               "")
-  return(paste(paste(px, commas, sep=""), collapse=""))
+  out <- paste0(px, commas, collapse=if(flatten) "" else NULL)
+  return(out)
 }
 
 paren <- function(x, type="(") {
@@ -191,6 +192,40 @@ unparen <- function(x) {
   return(x)
 }
 
+strsplitretain <- local({
+  strsplitretain <- function(x, split=",") {
+    ## split strings after occurrence of character b, but retain b
+    y <- strsplit(x, split)
+    lapply(y, addback, b=split)
+  }
+  addback <- function(x, b=",") {
+    n <- length(x)
+    if(n <= 1) x else c(paste0(x[-n], b), x[n])
+  }    
+  strsplitretain
+})
+
+truncline <- function(x, nc) {
+  if(length(x) > 1)
+    return(unlist(lapply(as.list(x), truncline, nc=nc)))
+  ## split string into words
+  y <- strsplit(x, " ", fixed=TRUE)[[1]]
+  ## find max number of whole words that take up nc characters
+  maxwords <- max(0, which(cumsum(nchar(y) + 1) <= nc+1))
+  if(maxwords == length(y))
+    return(x)
+  ## truncation will occur.
+  pad <- " [..]"
+  nc <- nc - nchar(pad)
+  maxwords <- max(0, which(cumsum(nchar(y) + 1) <= nc+1))
+  z <- paste(y[seq_len(maxwords)], collapse=" ")
+  d <- nc - nchar(z)
+  if(d < 0)
+    z <- substr(z, 1, nc)
+  z <- paste0(z, pad)
+  return(z)
+}
+
 fakecallstring <- function(fname, parlist) {
   cl <- do.call("call", append(list(name = fname), parlist))
   return(format(cl))
@@ -233,31 +268,52 @@ revcumsum <- function(x) {
   }
 }
 
-prolongseq <- function(x, newrange) {
+prolongseq <- function(x, newrange, step=NULL) {
+  ## Extend a sequence x so that it covers the new range.
   stopifnot(length(newrange) == 2 && newrange[1] < newrange[2])
-  stopifnot(length(x) >= 2)
-  dx <- diff(x)
-  if(any(dx <= 0))
-    stop("x must be an increasing sequence")
-  if(diff(range(dx)) > 0.01 * abs(mean(dx)))
-    stop("x must be evenly spaced")
-  dx <- mean(dx)
+  ## Check 'x' is an evenly-spaced sequence
+  if(length(x) > 1) {
+    dx <- diff(x)
+    if(any(dx <= 0))
+      stop("x must be an increasing sequence")
+    if(diff(range(dx)) > 0.01 * abs(mean(dx)))
+      stop("x must be evenly spaced")
+  }
+  ## Infer step length
+  if(!is.null(step)) {
+    check.1.real(step)
+    stopifnot(step > 0)
+  } else if(length(x) > 1) {
+    step <- mean(dx)
+  } else stop("step is needed when x is a single value")
 
-  # add or trim data to left
+  ## 
+  if(max(x) < newrange[1] || min(x) > newrange[2])
+    stop("x lies entirely outside the desired range")
+    
+  ## add or trim data to left
   if(x[1] > newrange[1]) {
-    leftbit <- seq(from=x[1], to=newrange[1], by= -dx) 
-    x <- c(rev(leftbit), x[-1])
-  } else 
+    leftbit <- seq(from=x[1], to=newrange[1], by= -step)[-1]
+    x <- c(rev(leftbit), x)
+    nleft <- length(leftbit)
+  } else {
+    nx <- length(x)
     x <- x[x >= newrange[1]]
+    nleft <- length(x) - nx
+  }
 
   # add or trim data to right
   nx <- length(x)
   if(newrange[2] > x[nx]) {
-    rightbit <- seq(from=x[nx], to=newrange[2], by= dx)
-    x <- c(x[-nx], rightbit)
-  } else 
+    rightbit <- seq(from=x[nx], to=newrange[2], by= step)[-1]
+    x <- c(x, rightbit)
+    nright <- length(rightbit)
+  } else {
     x <- x[x <= newrange[2]]
-
+    nright <- length(x) - nx
+  }
+  attr(x, "nleft") <- nleft
+  attr(x, "nright") <- nright
   return(x)
 }
 
@@ -278,11 +334,21 @@ inside.range <- function(x, r) {
 
 prettyinside <- function(x, ...) {
   r <- range(x, na.rm=TRUE)
+  if(diff(r) == 0) return(r[1])
   p <- pretty(x, ...)
   ok <- inside.range(p, r)
   return(p[ok])
 }
 
+prettydiscrete <- function(x, n=10) {
+  nx <- length(x)
+  dx <- nx %/% n
+  if(dx < 1) return(x)
+  i <- 1 + (0:(n-1)) * dx
+  return(x[i])
+}
+
+
 check.range <- function(x, fatal=TRUE) {
   xname <- deparse(substitute(x))
   if(identical(x, range(x, na.rm=TRUE)))
@@ -299,9 +365,6 @@ niceround <- function(x, m=c(1,2,5,10)) {
   return(z)
 }
 
-assign(".Spatstat.ProgressBar", NULL, envir = .spEnv)
-assign(".Spatstat.ProgressData", NULL, envir = .spEnv)
-
 progressreport <- function(i, n, every=min(100,max(1, ceiling(n/100))),
                            nperline=min(charsperline,
                              every * ceiling(charsperline /(every+3))),
@@ -316,17 +379,16 @@ progressreport <- function(i, n, every=min(100,max(1, ceiling(n/100))),
          txtbar={
            if(i == 1) {
              # initialise text bar
-             assign(".Spatstat.ProgressBar",
-                    txtProgressBar(1, n, 1, style=3),
-                    envir = .spEnv)
+             putSpatstatVariable("Spatstat.ProgressBar",
+                                 txtProgressBar(1, n, 1, style=3))
            } else {
              # get text bar
-             pbar <- get(".Spatstat.ProgressBar", envir = .spEnv)
+             pbar <- getSpatstatVariable("Spatstat.ProgressBar")
              # update 
              setTxtProgressBar(pbar, i)
              if(i == n) {
                close(pbar)
-               assign(".Spatstat.ProgressBar", NULL, envir = .spEnv)
+               putSpatstatVariable("Spatstat.ProgressBar", NULL)
              } 
            }
          },
@@ -341,13 +403,12 @@ progressreport <- function(i, n, every=min(100,max(1, ceiling(n/100))),
              }
              showtime <- FALSE
              showevery <- n
-             assign(".Spatstat.ProgressData",
-                    list(every=every, nperline=nperline,
-                         starttime=now,
-                         showtime=FALSE, showevery=n),
-                    envir=.spEnv)
+             putSpatstatVariable("Spatstat.ProgressData",
+                                 list(every=every, nperline=nperline,
+                                      starttime=now,
+                                      showtime=FALSE, showevery=n))
            } else {
-             pd <- get(".Spatstat.ProgressData", envir=.spEnv)
+             pd <- getSpatstatVariable("Spatstat.ProgressData")
              if(is.null(pd))
                stop(paste("progressreport called with i =", i, "before i = 1"))
              every     <- pd$every
@@ -376,11 +437,10 @@ progressreport <- function(i, n, every=min(100,max(1, ceiling(n/100))),
                      showevery <- min(niceround(aminute), showevery)
                  }
                }
-               assign(".Spatstat.ProgressData",
-                    list(every=every, nperline=nperline,
-                         starttime=starttime,
-                         showtime=showtime, showevery=showevery),
-                    envir=.spEnv)
+               putSpatstatVariable("Spatstat.ProgressData",
+                                   list(every=every, nperline=nperline,
+                                        starttime=starttime,
+                                        showtime=showtime, showevery=showevery))
              }
            }
            if(i == n) 
@@ -417,6 +477,28 @@ numalign <- function(i, nmax, zero="0") {
   return(out)
 }
 
+as2vector <- function(x) {
+  ## convert various wacky formats to numeric vector of length 2
+  ## for use as coordinates of a single point.
+  xname <- deparse(substitute(x))
+  if(is.numeric(x)) {
+    if(length(x) != 2)
+      stop(paste(xname, "should have length 2"))
+    return(x)
+  }
+  if(is.ppp(x)) {
+    if(npoints(x) != 1)
+      stop(paste(xname, "should consist of exactly one point"))
+    return(c(x$x, x$y))
+  }
+  if(is.list(x) && all(c("x", "y") %in% names(x))) {
+    if(length(x$x) != 1) stop(paste0(xname, "$x should have length 1"))
+    if(length(x$y) != 1) stop(paste0(xname, "$y should have length 1"))
+    return(c(x$x, x$y))
+  }
+  stop(paste("Format of", sQuote(xname), "not understood"))
+}
+
 ensure2vector <- function(x) {
   xname <- deparse(substitute(x))
   if(!is.numeric(x))
@@ -444,8 +526,8 @@ ensure3Darray <- function(x) {
   return(x)
 }
 
-check.nvector <- function(v, npoints, fatal=TRUE, things="data points",
-                          naok=FALSE) {
+check.nvector <- function(v, npoints=NULL, fatal=TRUE, things="data points",
+                          naok=FALSE, warn=FALSE) {
   # vector of numeric values for each point/thing
   vname <- sQuote(deparse(substitute(v)))
   whinge <- NULL
@@ -453,21 +535,26 @@ check.nvector <- function(v, npoints, fatal=TRUE, things="data points",
     whinge <- paste(vname, "is not numeric")
   else if(!is.atomic(v) || !is.null(dim(v)))  # vector with attributes
     whinge <- paste(vname, "is not a vector")
-  else if(length(v) != npoints)
+  else if(!is.null(npoints) && (length(v) != npoints))
     whinge <- paste("The length of", vname,
-                    "should equal the number of", things)
+                    paren(paste0("=", length(v))), 
+                    "should equal the number of", things,
+                    paren(paste0("=", npoints)))
   else if(!naok && any(is.na(v)))
     whinge <- paste("Some values of", vname, "are NA or NaN")
   #
   if(!is.null(whinge)) {
-    if(fatal) stop(whinge) else return(FALSE)
+    if(fatal) stop(whinge)
+    if(warn) warning(whinge)
+    return(FALSE)
   }
   return(TRUE)
 }
 
-check.nmatrix <- function(m, npoints, fatal=TRUE, things="data points",
-                          naok=FALSE, squarematrix=TRUE, matchto="nrow") {
-  # matrix of values for each thing or each pair of things
+check.nmatrix <- function(m, npoints=NULL, fatal=TRUE, things="data points",
+                          naok=FALSE, squarematrix=TRUE, matchto="nrow",
+                          warn=FALSE) {
+  ## matrix of values for each thing or each pair of things
   mname <- sQuote(deparse(substitute(m)))
   whinge <- NULL
   if(!is.matrix(m))
@@ -476,15 +563,23 @@ check.nmatrix <- function(m, npoints, fatal=TRUE, things="data points",
     whinge <- paste(mname, "should be a square matrix")
   else if(!naok && any(is.na(m)))
     whinge <- paste("Some values of", mname, "are NA or NaN")
-  else if(matchto=="nrow" && nrow(m) != npoints)
-    whinge <- paste("Number of rows in", mname,
-               "does not match number of", things)
-  else if(matchto=="ncol" && ncol(m) != npoints)
-    whinge <- paste("Number of columns in", mname,
-               "does not match number of", things)
-  #
+  else if(!is.null(npoints)) {
+    if(matchto=="nrow" && nrow(m) != npoints)
+      whinge <- paste("Number of rows in", mname,
+                      paren(paste0("=", nrow(m))),
+                      "does not match number of", things,
+                      paren(paste0("=", npoints)))
+    else if(matchto=="ncol" && ncol(m) != npoints)
+      whinge <- paste("Number of columns in", mname,
+                      paren(paste0("=", ncol(m))),
+                      "does not match number of", things,
+                      paren(paste0("=", npoints)))
+  }
+  ##
   if(!is.null(whinge)) {
-    if(fatal) stop(whinge) else return(FALSE)
+    if(fatal) stop(whinge)
+    if(warn) warning(whinge)
+    return(FALSE)
   }
   return(TRUE)
 }
@@ -542,6 +637,7 @@ check.named.thing <- function(x, nam, namopt=character(0), xtitle=NULL,
   return(whinge)
 }
 
+
 forbidNA <- function(x, context="", xname, fatal=TRUE, usergiven=TRUE) {
   if(missing(xname)) xname <- sQuote(deparse(substitute(x)))
   if(any(is.na(x))) {
@@ -768,6 +864,14 @@ nzpaste <- function(..., sep=" ", collapse=NULL) {
   do.call("paste", append(v[ok], list(sep=sep, collapse=collapse)))
 }
 
+substringcount <- function(x, y) {
+  ## count occurrences of 'x' in 'y'
+  yy <- paste0("a", y, "a")
+  splot <- strsplit(yy, split=x, fixed=TRUE)
+  nhits <- unlist(lapply(splot, length)) - 1
+  return(nhits)
+}
+
 is.parseable <- function(x) {
   unlist(lapply(x, function(z) {
     !inherits(try(parse(text=z), silent=TRUE), "try-error")
@@ -941,7 +1045,27 @@ dotexpr.to.call <- function(expr, dot="funX", evaluator="eval.fv") {
   return(cc)
 }
 
-# print names and version numbers of libraries loaded
+## Match variable names to objects in 'data' list or environment
+getdataobjects <- function(nama, envir, datalist=NULL) {
+  if(is.null(nama)) return(NULL)
+  stopifnot(is.character(nama))
+  n <- length(nama)
+  y <- vector(mode="list", length=n)
+  names(y) <- nama
+  if(!is.null(datalist)) {
+    hit <- nama %in% names(datalist)
+    if(any(hit))
+      y[hit] <- as.list(datalist)[nama[hit]]
+    needed <- unlist(lapply(y, is.null))
+  } else needed <- rep(TRUE, n)
+  y[needed] <- mget(nama[needed], envir=envir,
+                    ifnotfound=list(NULL), inherits=TRUE)
+  names(y) <- nama
+  attr(y, "external") <- needed
+  return(y)
+}
+ 
+## print names and version numbers of libraries loaded
 
 sessionLibs <- function() {
   a <- sessionInfo()
@@ -1060,3 +1184,59 @@ ifelseNegPos <- function(test, x) {
   # b is evaluated only now
   return(b)
 }
+
+blockdiagmatrix <- function(...) {
+  x <- list(...)
+  if(!all(unlist(lapply(x, is.matrix))))
+    stop("Some of the arguments are not matrices", call.=FALSE)
+  nr <- unlist(lapply(x, nrow))
+  nc <- unlist(lapply(x, ncol))
+  result <- matrix(0, sum(nr), sum(nc))
+  rownames(result) <- unlist(lapply(x, rownames))
+  colnames(result) <- unlist(lapply(x, colnames))
+  rowend <- cumsum(nr)
+  rowstart <- c(0, rowend) + 1
+  colend <- cumsum(nc)
+  colstart <- c(0, colend) + 1
+  for(i in seq_along(x))
+    result[ (rowstart[i]):(rowend[i]) , (colstart[i]):(colend[i])] <- x[[i]]
+  return(result)
+}
+
+blockdiagarray <- function(...) {
+  x <- list(...)
+  if(!all(unlist(lapply(x, is.array))))
+    stop("Some of the arguments are not arrays", call.=FALSE)
+  dims <- lapply(x, dim)
+  dims1 <- unlist(lapply(dims, "[", i=1))
+  if(length(dim1 <- unique(dims1)) > 1)
+    stop("Arrays have different extents in first dimension")
+  dims2 <- unlist(lapply(dims, "[", i=2))
+  dims3 <- unlist(lapply(dims, "[", i=3))
+  result <- array(0, dim=c(dim1, sum(dims2), sum(dims3)))
+  dn <- lapply(x, dimnames)
+  dimnames(result)[[2]] <- unlist(lapply(dn, "[[", 2))
+  dimnames(result)[[3]] <- unlist(lapply(dn, "[[", 3))
+  rowend <- cumsum(dims2)
+  rowstart <- c(0, rowend) + 1
+  colend <- cumsum(dims3)
+  colstart <- c(0, colend) + 1
+  for(i in seq_along(x))
+    result[ , (rowstart[i]):(rowend[i]) , (colstart[i]):(colend[i])] <- x[[i]]
+  return(result)
+}
+
+lty2char <- function(i) {
+  if(is.numeric(i)) c("blank", "solid", "dashed", "dotted",
+                      "dotdash", "longdash", "twodash")[(i %% 7) + 1] else i
+}
+
+## convert numeric matrix to character, and blank out lower sub-diagonal.
+uptrimat <- function(x) {
+  stopifnot(is.matrix(x))
+  x[] <- as.character(x)
+  x[row(x) > col(x)] <- ""
+  return(noquote(x))
+}
+
+                  
