@@ -3,7 +3,7 @@
 #
 #	A class 'owin' to define the "observation window"
 #
-#	$Revision: 4.135 $	$Date: 2013/11/12 09:30:10 $
+#	$Revision: 4.146 $	$Date: 2014/04/21 03:32:08 $
 #
 #
 #	A window may be either
@@ -93,12 +93,13 @@ owin <- function(xrange=c(0,1), yrange=c(0,1),
       poly <- lapply(poly, as.list)
   }
   
-  # avoid re-checking if already checked, or if checking suppressed
-  checkdefault <- spatstat.options("checkpolygons")
-  check <- resolve.defaults(list(...), list(check=checkdefault))$check
-
-  # whether to calculate polygon areas
-  calculate <- resolve.defaults(list(...), list(calculate=check))$calculate
+  ## Hidden options controlling how much checking is performed
+  check <- resolve.1.default(list(check=TRUE), list(...))
+  calculate <- resolve.1.default(list(calculate=check), list(...))
+  strict <- resolve.1.default(list(strict=spatstat.options("checkpolygons")),
+                              list(...))
+  fix <- resolve.1.default(list(fix=spatstat.options("fixpolygons")),
+                           list(...))
 
   if(!poly.given && !mask.given) {
     ######### rectangle #################
@@ -197,14 +198,29 @@ owin <- function(xrange=c(0,1), yrange=c(0,1),
     w <- list(type="polygonal",
               xrange=xrange, yrange=yrange, bdry=bdry, units=unitname)
     class(w) <- "owin"
-    # check for intersection or self-intersection
-    if(check) { 
+
+    if(check && strict) { 
+      ## strict checks on geometry (self-intersection etc)
       ok <- owinpolycheck(w)
       if(!ok) {
         errors <- attr(ok, "err")
         stop(paste("Polygon data contain", commasep(errors)))
       }
     }
+    if(check && fix) {
+      ## repair polygon data by invoking polyclip
+      ##        to intersect polygon with bounding rectangle
+      ##        (Streamlined version of intersect.owin)
+      ww <- lapply(bdry, reverse.xypolygon)
+      rr <- list(list(x=xrange[c(1,2,2,1)], y=yrange[c(2,2,1,1)]))
+      bb <- polyclip::polyclip(ww, rr, "intersection",
+                               fillA="nonzero", fillB="nonzero")
+      ## ensure correct polarity
+      totarea <- sum(unlist(lapply(bb, area.xypolygon)))
+      if(totarea < 0)
+        bb <- lapply(bb, reverse.xypolygon)
+      w$bdry <- bb
+    }
     return(w)
     
   } else if(mask.given) {
@@ -374,30 +390,33 @@ as.owin.data.frame <- function(W, ..., fatal=TRUE) {
 }
 
 as.owin.default <- function(W, ..., fatal=TRUE) {
-	# Tries to interpret data as an object of class 'window'
-	# W may be
-	#	a structure with entries xrange, yrange
-	#	a four-element vector (interpreted xmin, xmax, ymin, ymax)
-	#	a structure with entries xl, xu, yl, yu
+  ## Tries to interpret data as an object of class 'owin'
+  ## W may be
+  ##	a structure with entries xrange, yrange
+  ##	a four-element vector (interpreted xmin, xmax, ymin, ymax)
+  ##	a structure with entries xl, xu, yl, yu
+  ##	an object with attribute "bbox"
 
-        if(checkfields(W, c("xrange", "yrange"))) {
-		Z <- owin(W$xrange, W$yrange)
-		return(Z)
-	} else if(is.vector(W) && is.numeric(W) && length(W) == 4) {
-		Z <- owin(W[1:2], W[3:4])
-		return(Z)
-	} else if(checkfields(W, c("xl", "xu", "yl", "yu"))) {
-                W <- as.list(W)
-		Z <- owin(c(W$xl, W$xu),c(W$yl, W$yu))
-		return(Z)
-        } else if(checkfields(W, c("x", "y", "area"))
-                  && checkfields(W$area, c("xl", "xu", "yl", "yu"))) {
-                V <- as.list(W$area)
-                Z <- owin(c(V$xl, V$xu),c(V$yl, V$yu))
-                return(Z)
-	} else if(fatal)
-		stop("Can't interpret W as a window")
-        else return(NULL)
+  if(checkfields(W, c("xrange", "yrange"))) {
+    Z <- owin(W$xrange, W$yrange)
+    return(Z)
+  } else if(is.vector(W) && is.numeric(W) && length(W) == 4) {
+    Z <- owin(W[1:2], W[3:4])
+    return(Z)
+  } else if(checkfields(W, c("xl", "xu", "yl", "yu"))) {
+    W <- as.list(W)
+    Z <- owin(c(W$xl, W$xu),c(W$yl, W$yu))
+    return(Z)
+  } else if(checkfields(W, c("x", "y", "area"))
+            && checkfields(W$area, c("xl", "xu", "yl", "yu"))) {
+    V <- as.list(W$area)
+    Z <- owin(c(V$xl, V$xu),c(V$yl, V$yu))
+    return(Z)
+  } else if(!is.null(Z <- attr(W, "bbox"))) {
+    return(as.owin(Z, ..., fatal=fatal))
+  } else if(fatal)
+    stop("Can't interpret W as a window")
+  else return(NULL)
 }		
 
 #
@@ -409,6 +428,8 @@ as.rectangle <- function(w, ...) {
     return(owin(w$xrange, w$yrange, unitname=unitname(w)))
   else if(inherits(w, "im"))
     return(owin(w$xrange, w$yrange, unitname=unitname(w)))
+  else if(inherits(w, "layered")) 
+    return(do.call(boundingbox, unname(lapply(w, as.rectangle))))
   else {
     w <- as.owin(w, ...)
     return(owin(w$xrange, w$yrange, unitname=unitname(w)))
@@ -572,7 +593,8 @@ as.polygonal <- function(W) {
            xr <- W$xrange
            yr <- W$yrange
            return(owin(xr, yr, poly=list(x=xr[c(1,2,2,1)],y=yr[c(1,1,2,2)]),
-                       unitname=unitname(W)))
+                       unitname=unitname(W),
+                       check=FALSE))
          },
          polygonal = {
            return(W)
@@ -600,7 +622,7 @@ as.polygonal <- function(W) {
              if(ns != nf)
                stop(paste("Internal error: length(starts)=", ns,
                           ", length(finishes)=", nf))
-             if(ns > 0)
+             if(ns > 0) {
                for(k in 1:ns) {
                  yfrom <- yrow[starts[k]]
                  yto   <- yrow[finishes[k]]
@@ -610,6 +632,8 @@ as.polygonal <- function(W) {
                  # add to result
                  out <- union.owin(out, recto)
                }
+               unitname(out) <- unitname(W)
+             }
            }
            return(out)
          }
@@ -665,21 +689,21 @@ raster.xy <- function(w, drop=FALSE) {
 }
   
 nearest.raster.point <- function(x,y,w, indices=TRUE) {
-	validate.mask(w)
-	nr <- w$dim[1]
-	nc <- w$dim[2]
-        if(length(x) == 0) 
-          cc <- rr <- integer(0)
-        else {
-          cc <- 1 + round((x - w$xcol[1])/w$xstep)
-          rr <- 1 + round((y - w$yrow[1])/w$ystep)
-          cc <- pmax.int(1,pmin.int(cc, nc))
-          rr <- pmax.int(1,pmin.int(rr, nr))
-        }
-        if(indices) 
-          return(list(row=rr, col=cc))
-        else
-          return(list(x=w$xcol[cc], y=w$yrow[rr]))
+  stopifnot(is.mask(w) || is.im(w))
+  nr <- w$dim[1]
+  nc <- w$dim[2]
+  if(length(x) == 0) {
+    cc <- rr <- integer(0)
+  } else {
+    cc <- 1 + round((x - w$xcol[1])/w$xstep)
+    rr <- 1 + round((y - w$yrow[1])/w$ystep)
+    cc <- pmax.int(1,pmin.int(cc, nc))
+    rr <- pmax.int(1,pmin.int(rr, nr))
+  }
+  if(indices) 
+    return(list(row=rr, col=cc))
+  else
+    return(list(x=w$xcol[cc], y=w$yrow[rr]))
 }
 
 mask2df <- function(w) {
@@ -694,85 +718,6 @@ mask2df <- function(w) {
 
 #------------------------------------------------------------------
 		
-bounding.box <- function(...) {
-        wins <- list(...)
-
-        if(length(wins) == 0)
-          stop("No arguments supplied")
-
-        # trap a particular misuse of this function
-        if(length(wins) == 2) {
-          w1 <- wins[[1]]
-          w2 <- wins[[2]]
-          if(is.vector(w1) && is.numeric(w1) &&
-             is.vector(w2) && is.numeric(w2) &&
-             length(w1) == length(w2)) 
-            stop(paste("bounding.box() was applied to two numeric vectors;",
-                       "you probably wanted bounding.box.xy()"))
-        }
-
-        # remove null objects 
-        isnul <- unlist(lapply(wins, is.null))
-        if(all(isnul))
-          stop("All arguments are NULL")
-        wins <- wins[!isnul]
-        
-        if(length(wins) > 1) {
-          # multiple arguments -- compute bounding box for each argument.
-          # First trap any point patterns and extract bounding boxes of points
-          isppp <- unlist(lapply(wins, is.ppp))
-          if(any(isppp)) 
-            wins[isppp] <- lapply(wins[isppp], bounding.box.xy)
-          # then convert all windows to owin
-          wins <- lapply(wins, as.owin)
-          # then take bounding box of each window
-          boxes <- lapply(wins, bounding.box)
-          # discard NULL values
-          isnull <- unlist(lapply(boxes, is.null))
-          boxes <- boxes[!isnull]
-          # take bounding box of these boxes
-          xrange <- range(unlist(lapply(boxes, function(b){b$xrange})))
-          yrange <- range(unlist(lapply(boxes, function(b){b$yrange})))
-          return(owin(xrange, yrange))
-        }
-
-        # single argument
-        w <- wins[[1]]
-        if(is.null(w))
-          return(NULL)
-
-        # point pattern?
-        if(is.ppp(w))
-          return(bounding.box.xy(w))
-          
-        # convert to window
-        w <- as.owin(w)
-
-        # determine a tight bounding box for the window w
-        switch(w$type,
-               rectangle = {
-                 return(w)
-               },
-               polygonal = {
-                 bdry <- w$bdry
-                 if(length(bdry) == 0)
-                   return(NULL)
-                 xr <- range(unlist(lapply(bdry, function(a) range(a$x))))
-                 yr <- range(unlist(lapply(bdry, function(a) range(a$y))))
-                 return(owin(xr, yr, unitname=unitname(w)))
-               },
-               mask = {
-                 m <- w$m
-                 x <- raster.x(w)
-                 y <- raster.y(w)
-                 xr <- range(x[m]) + c(-1,1) * w$xstep/2
-                 yr <- range(y[m]) + c(-1,1) * w$ystep/2
-                 return(owin(xr, yr, unitname=unitname(w)))
-               },
-               stop("unrecognised window type", w$type)
-               )
-}
-  
 complement.owin <- function(w, frame=as.rectangle(w)) {
   wname <- short.deparse(substitute(w))
   w <- as.owin(w)
@@ -912,30 +857,31 @@ inside.owin <- function(x, y, w) {
 print.owin <- function(x, ...) {
   verifyclass(x, "owin")
   unitinfo <- summary(unitname(x))
-  cat("window: ")
   switch(x$type,
          rectangle={
-           cat("rectangle = ")
+           rectname <- "window: rectangle ="
          },
          polygonal={
-           cat("polygonal boundary\n")
+           cat("window:", "polygonal", "boundary", fill=TRUE)
            if(length(x$bdry) == 0)
              cat("window is empty\n")
-           cat("enclosing rectangle: ")
+           rectname <- "enclosing rectangle:"
          },
          mask={
-           cat("binary image mask\n")
+           cat("window: binary", "image mask", fill=TRUE)
            di <- x$dim
-           cat(paste(di[1], "x", di[2], "pixel array (ny, nx)\n"))
-           cat("enclosing rectangle: ")
+           cat(di[1], "x", di[2], "pixel array (ny, nx)", fill=TRUE)
+           rectname <- "enclosing rectangle:"
          }
          )
-    cat(paste(prange(zapsmall(x$xrange)),
-              "x",
-              prange(zapsmall(x$yrange)),
-              unitinfo$plural,
-              unitinfo$explain,
-              "\n"))
+  cat(rectname,
+      prange(zapsmall(x$xrange)),
+      "x",
+      prange(zapsmall(x$yrange)),
+      unitinfo$plural,
+      unitinfo$explain,
+      fill=TRUE)
+  invisible(NULL)
 }
 
 summary.owin <- function(object, ...) {
@@ -978,34 +924,36 @@ print.summary.owin <- function(x, ...) {
   unitinfo <- summary(x$units)
   pluralunits <- unitinfo$plural
   singularunits <- unitinfo$singular
-  cat("Window: ")
   switch(x$type,
          rectangle={
-           cat("rectangle = ")
+           rectname <- "Window: rectangle ="
          },
          polygonal={
-           cat("polygonal boundary\n")
-           if(x$npoly == 0) {
+           np <- x$npoly
+           cat("Window:", "polygonal", "boundary", fill=TRUE)
+           if(np == 0) {
              cat("window is empty\n")
-           } else if(x$npoly == 1) {
-             cat(paste("single connected closed polygon with",
-                       x$nvertices, 
-                       "vertices\n"))
+           } else if(np == 1) {
+             cat("single connected", "closed polygon",
+                 "with",
+                 x$nvertices, 
+                 "vertices", fill=TRUE)
            } else {
-             cat(paste(x$npoly, "separate polygons ("))
-             if(x$nhole == 0) cat("no holes)\n")
-             else if(x$nhole == 1) cat("1 hole)\n")
-             else cat(paste(x$nhole, "holes)\n"))
-             if(x$npoly > 0)
+             nh <- x$nhole
+             holy <- if(nh == 0) "(no holes)" else
+                     if(nh == 1) "(1 hole)" else
+                     paren(paste(nh, "holes"))
+             cat(np, "separate polygons", holy, fill=TRUE)
+             if(np > 0)
                print(data.frame(vertices=x$nvertices,
                                 area=signif(x$areas, 6),
                                 relative.area=signif(x$areas/x$area,3),
                                 row.names=paste("polygon",
-                                  1:(x$npoly),
+                                  1:np,
                                   ifelse(x$areas < 0, "(hole)", "")
                                   )))
            }
-           cat("enclosing rectangle: ")
+           rectname <- "enclosing rectangle:"
          },
          mask={
            cat("binary image mask\n")
@@ -1014,18 +962,21 @@ print.summary.owin <- function(x, ...) {
            cat(paste("pixel size:",
                      signif(x$xstep,3), "by", signif(x$ystep,3),
                      pluralunits, "\n"))
-           cat("enclosing rectangle: ")
+           rectname <- "enclosing rectangle:"
          }
          )
-  cat(paste(prange(zapsmall(x$xrange)),
-            "x",
-            prange(zapsmall(x$yrange)),
-            pluralunits, "\n"))
+  cat(rectname,
+      prange(zapsmall(x$xrange)),
+      "x",
+      prange(zapsmall(x$yrange)),
+      pluralunits, 
+      fill=TRUE)
   Area <- signif(x$area, 6)
-  cat(paste("Window area = ", Area, "square",
-            if(Area == 1) singularunits else pluralunits, "\n"))
+  cat("Window area =", Area, "square",
+      if(Area == 1) singularunits else pluralunits,
+      fill=TRUE)
   if(!is.null(ledge <- unitinfo$legend))
-    cat(paste(ledge, "\n"))
+    cat(ledge, "\n")
   return(invisible(x))
 }
 
@@ -1047,3 +998,10 @@ discretise <- function(X,eps=NULL,dimyx=NULL,xy=NULL) {
   X$window <- WM
   X
 }
+
+pixelcentres <- function (X, W=NULL,...) {
+  X <- as.mask(as.owin(X), ...)
+  if(is.null(W)) W <- as.rectangle(X)
+  Y <- as.ppp(raster.xy(X,drop=TRUE),W=W)
+  return(Y)
+}
